! **********************************************************************
! *                                                                    *
! *     1-D Half-space cooling model for plate thermal evolution       *
! *               solved by finite difference method                   *
! *                                                                    *
! **********************************************************************

      program plate_hscm

      implicit none

      real(8), parameter::   SecY = 365.2422d0*24.0d0*3600.0d0
      real(8), allocatable:: Tmp(:),Tnew(:)
      real(8):: rho,Cp,ak,D
      real(8):: age,T0,TM
      real(8):: z,zk,zmax,zkmax,dz,dzk,dz2
      real(8):: t,tmax,tevo,dt,dtmy,dtlim
      integer:: j,nz
      integer:: it,ntmax
      integer:: nf
      integer:: itskp1,itskp2
      character(40):: apara

!  *****  read parameters  *****

!  Open file

      open ( 10, file = 'pl_para_2.dat' )

!  read parameter file 

      read (10,*) apara
      read (10,*) rho
      write (6,*) apara,' = ',rho

      read (10,*) apara
      read (10,*) Cp
      write (6,*) apara,' = ',Cp

      read (10,*) apara
      read (10,*) ak
      write (6,*) apara,' = ',ak

      read (10,*) apara
      read (10,*) age
      write (6,*) apara,' = ',age

      read (10,*) apara
      read (10,*) T0
      write (6,*) apara,' = ',T0

      read (10,*) apara
      read (10,*) TM
      write (6,*) apara,' = ',TM

      read (10,*) apara
      read (10,*) nz
      write (6,*) apara,' = ',nz

      read (10,*) apara
      read (10,*) zkmax
      write (6,*) apara,' = ',zkmax

      read (10,*) apara
      read (10,*) dtmy
      write (6,*) apara,' = ',dtmy

      read (10,*) apara
      read (10,*) itskp1
      write (6,*) apara,' = ',itskp1

      read (10,*) apara
      read (10,*) itskp2
      write (6,*) apara,' = ',itskp2

!  convert unit km -> m, Ma -> sec

      zmax = zkmax  * 1.0d3
      tmax = age  * 1.0d6 * SecY
      dt   = dtmy * 1.0d6 * SecY

!  dz: grid size  ****

      dz   = zmax / dfloat( nz )
      dz2  = dz * dz

!  maximum time step number

      ntmax = int( tmax/dt + 0.5d0 ) 

!  thermal diffusivity

      D = ak / ( rho*Cp )

!  *****  check dt for numerical stability  ****

      dtlim = 0.5d0 * dz**2 / D
      if ( dt.gt.dtlim ) then
        dt = dtlim * 0.5d0
        write (6,*) '******  caution!!  time step is changed  ******'
      end if

!  allocate dimension variables

      allocate ( Tmp(0:nz), Tnew(nz) )

!  ****  Set initial condition  ****

      it = 0
      t  = 0.0d0

      Tmp(0)  = T0
      do j = 1,nz
        Tmp(j) = TM
      end do

!  *** write initial condition to file ***

      nf = 0

      call writefile( Tmp, dz, nz, nf )

!  ***  Open file for haet flux  ****

      open ( 20,file = 'hf2.dat',status='new' )
      call      heatflux( Tmp(0), Tmp(1), t, ak, dz )

!  ==========  Loop of time marching  ================================

      do it = 1,ntmax

!  ***  Time ****

        t  = dt * dfloat( it )

! ***  Time marching by Euler method  ***

        do j = 1,nz-1
          Tnew(j) = Tmp(j) + dt * D / dz2                            &
                          *( Tmp(j-1)+Tmp(j+1)-2.0d0*Tmp(j) )
        end do

!  ***  Change old and new temperatures

        do j = 1,nz-1
          Tmp(j) = Tnew(j)
        end do

!  ***  Boundary condition: dT/dz=0 at bottom  ***

        Tmp(nz) = Tmp(nz-1)

!  ***  Write results to file in each itskp step  ***

!        if ( mod(it,itskp1).eq.1 .or. itskp1.eq.1 ) then
        if ( mod(it,itskp1).eq.0 ) then
          tevo = t / SecY /1.0d6
          write (6,*) it,tevo
          nf = nf + 1
          call  writefile( Tmp, dz, nz, nf )
        end if

!  ***  calculate heat flux and write to file in each itskp2 step  ***

!        if ( mod(it,itskp2).eq.0 .or. itskp2.eq.1 )                  &
        if ( mod(it,itskp2).eq.0 )                  &
          call  heatflux( Tmp(0), Tmp(1), t, ak, dz ) 

      end do

!  =================  End of the loop  ===============================

      close (20)
      stop
      end program plate_hscm

